# 2020-08-10 - added command to kill marble items when stopping game by putting a marble in the off hand and corner physics
# 2020-08-11 - changed timing used with the Age tag, made stand marble not be able to be picked until item as despawned, change when marker stand is killed
# 2020-08-12 - Many changes to allow doing courses where the marble can fall from blocks and first section of instructions
# 2020-08-13 - Added other instructions (minus already existing "adding more marbles") and credits
# 2020-08-14 - Now the tag "playing_marbles" is used mostly for marbles to stop playing players from influencing also marbles outside playfield
# 2020-08-16 - Added video link

# WHAT TO CHANGE FOR YOUR CUSTOM TRACK
# 1 - Change the playable area ("x=293,y=62,z=418,dx=17,dy=5,dz=8") or choose another way to give to the marbles the tag "playing_marbles"
# 2 - Decide if making the players start from the same position
# 3 - If you didn't build the track on sand, change the block on all the lines that have it.
# If you want to make the players start from the same position, summon an armor stand with the tag "marble_end" on each end of the course
# The armor stand must be summoned exactly on the y value the marbles start from
# For example, if the track has two ends made by a slab at the y position 65:
# /summon armor_stand 295 64.5 419 {Marker:true,Invisible:true,NoGravity:true,Tags:["course_end1","marble_end"]}
# /summon armor_stand 294 64.5 422 {Marker:true,Invisible:true,NoGravity:true,Tags:["course_end2","marble_end"]}
# As you can see, in this case the armor stand will be summoned at the y position 64.5, because the player will stand on a slab

# HOW TO BUILD A MARBLE COURSE:
# A course is composed of a track, the in-bounds portion, and the out-of-bounds portion, that includes the walls of the track
# Under the track, there should be ice to make the marbles slide; decide what type of ice using (blue ice makes items slide the most), or don't, if you want items to don't slide
# To make the ice actually do something, the tracks must be made of slabs (any kind), but you can also decide to make the marbles slide directly on the ice
# The out of bounds block is used by some of the command, by default sand, so you have to change those commands if you don't use sand
# You can use a custom block tag if you don't want to be limited to one block
# In the design of the track, you're not limited to a single plane, the track can have drops
# Thrown marbles are actually armor stands: the item they carry gets dark if the stand is inside not transparent blocks, so try to use them instead of opaque ones

# ADDING MORE MARBLES
# In this file, look for every command which is repeated twice: for each new marble, you have to add another command of those.
# Every new marble item must have two tags: "marble" and an unique one for each marble (in this file, "magma" and "slime" are used).

# ITEMS MIGHT GLITCH:
# Items in Minecraft are still pretty glitchy, after all this time
# Many commands in this track were carefully written to account for that, but they might still fail sometimes
# A possible occurrence is that the marble might go inside a block, disappearing inside the ground
# Putting a marble inside your off hand slot will always reset the game, so do that if something like that happens to recover all marbles
# You could also decide to use a command like this:
#execute as @e[type=item,nbt={Item:{tag:{marble:true}}},tag=playing_marbles] at @s unless block ~ ~0.5 ~ air run data modify entity @s Pos[1] set from entity @e[tag=marble_end,limit=1,sort=nearest] Pos[1]
# This command would teleport any marble that went in the ground to the surface, using the y position of the nearest armor stand used for the ends of the track
# Still, the command doesn't always work and can produce some jarring visual glitches, so use it carefully


# HOW TO PLAY:
# If the course has functional ends, take a marble, put it in your hand and stand on an end to be locked in position; if not, you can start from any position
# Throw the marble inside the track: it will slide for 4 seconds, or until it's stopped by a wall
# If the marble reaches a corner, it will be influenced by it and move to the side
# When the marble has stopped, you can right click it to get it back: you will now be locked in the position you picked it up and you can do your next throw
# You will be locked in that position as long as you hold the marble, so you'll be freed if you don't
# If you throw the marble out of bounds, the throw will be cancelled: pick the marble back and you can now throw again from the last position on the track
# To stop the game at any time, put a marble in your off hand slot and you will get back every marble left on the track
# To stop playing, leave the area of the course: outside of it, the marble will behave most like any other item


# General commands

# 2020-08-10 - Only marbles with the tag "playing_marbles" can play
# 2020-08-14 - The tag is given to any entity and a command checks the tag also on the players
# The first command removes the tag if the next commands isn't activating
tag @e remove playing_marbles
# This command gives the tag if the marbles are inside the area of the marble course; you can decide to give this tag in other ways
tag @e[x=293,y=62,z=418,dx=17,dy=5,dz=8] add playing_marbles

# 2020-08-10 - Only one marble of the same type can exist at any moment
# The first thrown marble is given the tag "marble_exist"
tag @e[type=item,nbt={Item:{tag:{marble:true}},Age:1s}] add marble_exist
# Any subsequent marble of the same type will be impossible to throw
# These commands will activate if there is already a marble item
execute if entity @e[type=item,nbt={Item:{tag:{marble:true,magma:true}}},tag=marble_exist] as @e[type=item,nbt={Item:{tag:{marble:true,magma:true}}},tag=!marble_exist,tag=playing_marbles] at @s run data merge entity @s {PickupDelay:0}
execute if entity @e[type=item,nbt={Item:{tag:{marble:true,slime:true}}},tag=marble_exist] as @e[type=item,nbt={Item:{tag:{marble:true,slime:true}}},tag=!marble_exist,tag=playing_marbles] at @s run data merge entity @s {PickupDelay:0}
# These commands will activate if there is already a stand marble
execute if entity @e[tag=marble,tag=magma] as @e[type=item,nbt={Item:{tag:{marble:true,magma:true}}},tag=!marble_exist] at @s run data merge entity @s {PickupDelay:0}
execute if entity @e[tag=marble,tag=slime] as @e[type=item,nbt={Item:{tag:{marble:true,slime:true}}},tag=!marble_exist] at @s run data merge entity @s {PickupDelay:0}

# 2020-08-10 - The marble item model is changed to an invisible model when the stand marble is spawned so that both can't be seen at the same time
execute as @e[type=item,nbt={Item:{tag:{marble:true,magma:true}},Age:1s},tag=playing_marbles] at @s run data merge entity @s {Item:{tag:{CustomModelData:1,invisible:true},PickupDelay:32767}}
execute as @e[type=item,nbt={Item:{tag:{marble:true,slime:true}},Age:1s},tag=playing_marbles] at @s run data merge entity @s {Item:{tag:{CustomModelData:1,invisible:true},PickupDelay:32767}}

# 2020-08-12 - If, right before being killed, the marble is still in the air, this command will delay its death until it touches ground
# 2020-08-13 - If you 're playing over void, the marble will fall down and despawn, so account for that and add/change some commands
execute as @e[type=item,nbt={Item:{tag:{marble:true}},Age:80s,OnGround:false}] run data modify entity @s Age set value 79

# 2020-08-12 - If a marble isn't used while playing, it's changed to not despawn automatically
execute as @e[type=item,nbt={Item:{tag:{marble:true}},Age:1s},tag=!playing_marbles] run data modify entity @s Age set value -32768


# Keeps thrown marbles on the ground
data modify entity @e[limit=1,type=item,nbt={Item:{tag:{marble:true}}}] Motion[1] set value -1.0
# Makes the marbles inside the playable area unable to be picked up
execute as @e[type=item,nbt={Item:{tag:{marble:true}},Age:2s},tag=playing_marbles] run data modify entity @s PickupDelay set value 32767
# Creates a stand marble when it has been thrown
execute as @e[type=item,nbt={Item:{tag:{marble:true,magma:true}},Age:1s},tag=playing_marbles] at @s run summon armor_stand ~ ~-1.855 ~ {ArmorItems:[{},{},{},{id:magma_cream,Count:1b,tag:{"display":{"Name":"[{\"text\":\"Marble\",\"italic\":false,\"bold\":true}]","Lore":["{\"text\":\"Throw it the farthest you can\",\"color\":\"gray\",\"bold\":false,\"italic\":true}","{\"text\":\"Put it in your off-hand to stop the game\",\"color\":\"gray\",\"bold\":false,\"italic\":true}"]},CustomModelData:2,marble:true,magma:true}}],NoGravity:true,Invisible:true,Tags:["marble","marble_reset","magma"]}
execute as @e[type=item,nbt={Item:{tag:{marble:true,slime:true}},Age:1s},tag=playing_marbles] at @s run summon armor_stand ~ ~-1.855 ~ {ArmorItems:[{},{},{},{id:slime_ball,Count:1b,tag:{"display":{"Name":"[{\"text\":\"Marble\",\"italic\":false,\"bold\":true}]","Lore":["{\"text\":\"Throw it the farthest you can\",\"color\":\"gray\",\"bold\":false,\"italic\":true}","{\"text\":\"Put it in your off-hand to stop the game\",\"color\":\"gray\",\"bold\":false,\"italic\":true}"]},CustomModelData:2,marble:true,slime:true}}],NoGravity:true,Invisible:true,Tags:["marble","marble_reset","slime"]}
# 2020-08-11 - Marble can't be picked from stand while the marble item still exist
# The first command constantly makes the marble removable
execute as @e[tag=marble] run data modify entity @s DisabledSlots set value 3855
# If the marble item is still around, these commands will override the previous one and make the marble not removable
execute if entity @e[type=item,nbt={Item:{tag:{marble:true,magma:true}}},tag=playing_marbles] as @e[tag=marble,tag=magma] run data modify entity @s DisabledSlots set value 7967
execute if entity @e[type=item,nbt={Item:{tag:{marble:true,slime:true}}},tag=playing_marbles] as @e[tag=marble,tag=slime] run data modify entity @s DisabledSlots set value 7967
# Kills the marble item after the stand marble has been created
# 2020-08-14 - Changed command to kill the marble only if its stand marble exists and if it can age
execute as @e[type=item,nbt={Item:{tag:{marble:true}},Age:81s},tag=playing_marbles] at @s positioned ~ ~-1.855 ~ if entity @e[tag=marble,distance=..0] unless entity @s[nbt={Age:-32768s}] run kill @s
# 2020-08-11 - Invisible marbles are always killed 4 seconds after they have been summoned
execute as @e[type=item,nbt={Item:{tag:{marble:true,invisible:true}},Age:81s}] run kill @s


#Throwing mechanism's commands

# Teleports the player to the marble's position once the stand marble has been picked up
execute as @e[tag=marble,tag=magma] at @s positioned ~ ~1.96 ~ unless entity @s[nbt={ArmorItems:[{},{},{},{Count:1b,tag:{marble:true,magma:true}}]}] unless entity @e[tag=marble_player,tag=magma] run tp @a[x=293,y=62,z=418,dx=17,dy=5,dz=8,nbt={SelectedItem:{tag:{marble:true,magma:true}}}] ~ ~ ~
execute as @e[tag=marble,tag=slime] at @s positioned ~ ~1.96 ~ unless entity @s[nbt={ArmorItems:[{},{},{},{Count:1b,tag:{marble:true,slime:true}}]}] unless entity @e[tag=marble_player,tag=slime] run tp @a[x=293,y=62,z=418,dx=17,dy=5,dz=8,nbt={SelectedItem:{tag:{marble:true,slime:true}}}] ~ ~ ~
# 2020-08-10 - changed the commands so that the player can play also if holding a stack of the same marble while one has been launched
# Spawns the stand to which the player will be constantly teleported until they have launched the marble.
# This marker has gravity to account for the two possible positions the player could be teleported to: the track, a half slab, or the border, a full block.
execute as @e[tag=marble,tag=magma] at @s unless entity @s[nbt={ArmorItems:[{},{},{},{Count:1b,tag:{marble:true,magma:true}}]}] unless entity @e[tag=marble_player,tag=magma] run summon armor_stand ~ ~2.355 ~ {NoGravity:false,Invisible:true,Tags:["marble_player","marble_reset","magma"]}
execute as @e[tag=marble,tag=slime] at @s unless entity @s[nbt={ArmorItems:[{},{},{},{Count:1b,tag:{marble:true,slime:true}}]}] unless entity @e[tag=marble_player,tag=slime] run summon armor_stand ~ ~2.355 ~ {NoGravity:false,Invisible:true,Tags:["marble_player","marble_reset","slime"]}
# Teleports the player to the now spawned stand marker
execute at @e[tag=marble_player,tag=magma] unless entity @e[tag=marble,tag=magma,nbt={ArmorItems:[{},{},{},{Count:1b,tag:{marble:true,magma:true}}]}] run tp @a[tag=playing_marbles,nbt={SelectedItem:{tag:{marble:true,magma:true}}}] ~ ~ ~
execute at @e[tag=marble_player,tag=slime] unless entity @e[tag=marble,tag=slime,nbt={ArmorItems:[{},{},{},{Count:1b,tag:{marble:true,slime:true}}]}] run tp @a[tag=playing_marbles,nbt={SelectedItem:{tag:{marble:true,slime:true}}}] ~ ~ ~
# Kill the marker stand of the previous throw when the marble item thrown in the next throw is about to despawn (before it was killed before the next throw)
execute as @e[limit=1,type=item,nbt={Item:{tag:{marble:true,magma:true}},Age:80s}] run kill @e[tag=marble_player,tag=magma]
execute as @e[limit=1,type=item,nbt={Item:{tag:{marble:true,slime:true}},Age:80s}] run kill @e[tag=marble_player,tag=slime]

# Stops players from cheating by throwing the marble on the blocks that are not part of the track
# 2020-08-11 - Now it works like when the game is stopped by putting a marble in the offhand slot
# 2020-08-12 - Now, instead of being given back directly to the player, it's summoned as an entity with a pick up delay of 1 second, forcing the player to pick it up themselves
# 2020-08-14 - 
execute as @e[type=item,nbt={Item:{tag:{marble:true,magma:true}}},tag=playing_marbles] at @s if block ~ ~-1 ~ sand if entity @e[tag=marble,tag=magma,nbt={ArmorItems:[{},{},{},{Count:1b,tag:{marble:true,magma:true}}]}] at @e[tag=marble,tag=magma] run summon item ~ ~2 ~ {Age:-32768s,PickupDelay:20s,Item:{id:magma_cream,Count:1b,tag:{"display":{"Name":"[{\"text\":\"Marble\",\"italic\":false,\"bold\":true}]","Lore":["{\"text\":\"Throw it the farthest you can\",\"color\":\"gray\",\"bold\":false,\"italic\":true}","{\"text\":\"Put it in your off-hand to stop the game\",\"color\":\"gray\",\"bold\":false,\"italic\":true}"]},CustomModelData:2,marble:true,magma:true}}}
execute as @e[type=item,nbt={Item:{tag:{marble:true,slime:true}}},tag=playing_marbles] at @s if block ~ ~-1 ~ sand if entity @e[tag=marble,tag=slime,nbt={ArmorItems:[{},{},{},{Count:1b,tag:{marble:true,slime:true}}]}] at @e[tag=marble,tag=slime] run summon item ~ ~2 ~ {Age:-32768s,PickupDelay:20s,Item:{id:slime_ball,Count:1b,tag:{"display":{"Name":"[{\"text\":\"Marble\",\"italic\":false,\"bold\":true}]","Lore":["{\"text\":\"Throw it the farthest you can\",\"color\":\"gray\",\"bold\":false,\"italic\":true}","{\"text\":\"Put it in your off-hand to stop the game\",\"color\":\"gray\",\"bold\":false,\"italic\":true}"]},CustomModelData:2,marble:true,slime:true}}}
execute as @e[type=item,nbt={Item:{tag:{marble:true,magma:true}}},tag=playing_marbles] at @s if block ~ ~-1 ~ sand run kill @e[tag=marble,tag=magma]
execute as @e[type=item,nbt={Item:{tag:{marble:true,slime:true}}},tag=playing_marbles] at @s if block ~ ~-1 ~ sand run kill @e[tag=marble,tag=slime]
execute as @e[type=item,nbt={Item:{tag:{marble:true}}},tag=playing_marbles] at @s if block ~ ~-1 ~ sand unless entity @s[nbt={Age:-32768s}] run kill @s


# 2020-08-10
# Corner Physics: marbles are influenced by the corners to go a bit to the side
# These commands constantly teleport the stand marble to its corresponding type of marble item
execute as @e[type=item,nbt={Item:{tag:{marble:true,magma:true}}},tag=marble_exist] at @s positioned ~ ~-1.855 ~ run tp @e[tag=marble,tag=magma,limit=1,sort=nearest] ~ ~ ~
execute as @e[type=item,nbt={Item:{tag:{marble:true,slime:true}}},tag=marble_exist] at @s positioned ~ ~-1.855 ~ run tp @e[tag=marble,tag=slime,limit=1,sort=nearest] ~ ~ ~
# The marble item rotation is changed to be the same one as the player
execute as @e[type=item,nbt={Item:{tag:{marble:true}},Age:0s}] at @s run data modify entity @s Rotation[0] set from entity @p Rotation[0]
# Each type of stand marble copies the rotation of the marble item, getting as well the player's orientation
execute as @e[tag=marble,tag=magma] at @s run data modify entity @s Rotation[0] set from entity @e[limit=1,sort=nearest,type=item,nbt={Item:{tag:{marble:true,magma:true}}}] Rotation[0]
execute as @e[tag=marble,tag=slime] at @s run data modify entity @s Rotation[0] set from entity @e[limit=1,sort=nearest,type=item,nbt={Item:{tag:{marble:true,slime:true}}}] Rotation[0]

# To get the right influence from the corner, the stand marble can get 4 tags, one corresponding to a wall on each cardinal direction
# First, these commands remove the tags if the marble is not near each of the corresponding wall
tag @e[type=item,nbt={Item:{tag:{marble:true}}}] remove marble_wall_E
tag @e[type=item,nbt={Item:{tag:{marble:true}}}] remove marble_wall_W
tag @e[type=item,nbt={Item:{tag:{marble:true}}}] remove marble_wall_S
tag @e[type=item,nbt={Item:{tag:{marble:true}}}] remove marble_wall_N
# If the marble has a wall near it, it will get the corresponding tag
# The distance is 0.9, instead of 1.0, to stop the marble from sometime glitching and go in the ground; it also makes a bit more sense realistically
execute as @e[type=item,nbt={Item:{tag:{marble:true}}}] at @s if block ~0.9 ~ ~ sand run tag @s add marble_wall_E
execute as @e[type=item,nbt={Item:{tag:{marble:true}}}] at @s if block ~-0.9 ~ ~ sand run tag @s add marble_wall_W
execute as @e[type=item,nbt={Item:{tag:{marble:true}}}] at @s if block ~ ~ ~0.9 sand run tag @s add marble_wall_S
execute as @e[type=item,nbt={Item:{tag:{marble:true}}}] at @s if block ~ ~ ~-0.9 sand run tag @s add marble_wall_N
# Depending on the corner and the direction the marble is coming from, it will be teleport slightly to one side
execute as @e[type=item,nbt={Item:{tag:{marble:true}}},tag=marble_wall_E,tag=marble_wall_S,y_rotation=-45..45] at @s rotated 0 0 run tp @s ^-0.01 ^ ^
execute as @e[type=item,nbt={Item:{tag:{marble:true}}},tag=marble_wall_E,tag=marble_wall_S,y_rotation=-135..-45] at @s rotated -90 0 run tp @s ^0.01 ^ ^
execute as @e[type=item,nbt={Item:{tag:{marble:true}}},tag=marble_wall_E,tag=marble_wall_N,y_rotation=-135..-45] at @s rotated -90 0 run tp @s ^-0.01 ^ ^
execute as @e[type=item,nbt={Item:{tag:{marble:true}}},tag=marble_wall_E,tag=marble_wall_N,y_rotation=135..-135] at @s rotated 180 0 run tp @s ^0.01 ^ ^
execute as @e[type=item,nbt={Item:{tag:{marble:true}}},tag=marble_wall_W,tag=marble_wall_S,y_rotation=-45..45] at @s rotated 0 0 run tp @s ^0.01 ^ ^
execute as @e[type=item,nbt={Item:{tag:{marble:true}}},tag=marble_wall_W,tag=marble_wall_S,y_rotation=45..135] at @s rotated 90 0 run tp @s ^-0.01 ^ ^
execute as @e[type=item,nbt={Item:{tag:{marble:true}}},tag=marble_wall_W,tag=marble_wall_N,y_rotation=135..-135] at @s rotated 180 0 run tp @s ^-0.01 ^ ^
execute as @e[type=item,nbt={Item:{tag:{marble:true}}},tag=marble_wall_W,tag=marble_wall_N,y_rotation=45..135] at @s rotated 90 0 run tp @s ^0.01 ^ ^
# Removed idea: change also marble's rotation to be in the direction it's being teleported, so that it can influenced by the next corner (ex: add 90 ~ at end  of the 1st command)
# Removed because too glitchy, given the nature of Minecraft's items

# 2020-08-11
# Invisible marbles are always made impossible to be picked up
execute as @e[type=item,nbt={Item:{tag:{marble:true,invisible:true}}}] run data modify entity @s PickupDelay set value 32767
# In case an invisible marble gets in the player inventory, if they are not in creative, it will be removed
clear @a[gamemode=!creative] magma_cream{tag:{invisible:true}}
clear @a[gamemode=!creative] slime_ball{tag:{invisible:true}}
# This command will keep the stand marble bright
# The command is not active because Minecraft still doesn't have a way to remove the flames effect from entities on fire...
#execute as @e[tag=marble] run data merge entity @s {Fire:100s}



# START and STOP

# 2020-08-12 - Changed from being area based to being based on an armor stand
# This makes sure players start from the exact same point, but you can freely turn it off if it's annoying or you don't want to spawn more armor stands
# By standing on one end of the track and holding the marble, the game can be started
execute as @e[tag=marble_end] at @s run tp @a[dx=0,dy=0,dz=0,nbt={SelectedItem:{tag:{marble:true}}}] ~ ~ ~

# To stop the game, a player has to hold in their offhand the marble (to do so, by default, press F while holding the marble)
# If there are stand marble on the ground when the game is stopped, they will be given back
execute if entity @a[nbt={Inventory:[{Slot:-106b,tag:{marble:true}}]},tag=playing_marbles] if entity @e[tag=marble,tag=magma,nbt={ArmorItems:[{},{},{},{Count:1b,tag:{marble:true,magma:true}}]}] run give @a magma_cream{"display":{"Name":"[{\"text\":\"Marble\",\"italic\":false,\"bold\":true}]","Lore":["{\"text\":\"Throw it the farthest you can\",\"color\":\"gray\",\"bold\":false,\"italic\":true}","{\"text\":\"Put it in your off-hand to stop the game\",\"color\":\"gray\",\"bold\":false,\"italic\":true}"]},CustomModelData:2,marble:true,magma:true}
execute if entity @a[nbt={Inventory:[{Slot:-106b,tag:{marble:true}}]},tag=playing_marbles] if entity @e[tag=marble,tag=slime,nbt={ArmorItems:[{},{},{},{Count:1b,tag:{marble:true,slime:true}}]}] run give @a slime_ball{"display":{"Name":"[{\"text\":\"Marble\",\"italic\":false,\"bold\":true}]","Lore":["{\"text\":\"Throw it the farthest you can\",\"color\":\"gray\",\"bold\":false,\"italic\":true}","{\"text\":\"Put it in your off-hand to stop the game\",\"color\":\"gray\",\"bold\":false,\"italic\":true}"]},CustomModelData:2,marble:true,slime:true}
# Marble items that have spawned a stand marble will get the tag used by the next /kill command
# 2020-08-14 - Changed to be more precise and checking for stand marble instead of Age=1
execute as @e[type=item,nbt={Item:{tag:{marble:true}}},tag=playing_marbles] at @s positioned ~ ~-1.855 ~ if entity @e[tag=marble,distance=..0] run tag @s add marble_reset
# Kills every entity used in the game
# 2020-08-14 - Changed to not kill marbles that don't age
execute if entity @a[nbt={Inventory:[{Slot:-106b,tag:{marble:true}}]},tag=playing_marbles] as @e[tag=marble_reset] run kill @s[nbt=!{Age:-32768s}]

# When the player picks up the stand marble and gets the marble item, the stand is killed
execute as @e[tag=marble] unless entity @s[nbt={ArmorItems:[{},{},{},{Count:1b,tag:{marble:true}}]}] run kill @s



# Video about this datapack: https://youtu.be/vewLn8JHkwQ
# This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International License.
# You can monetize videos about this datapack; for any other exception, like modifying the datapack, you must first contact the creator.
# This data pack can be used freely; if you do intend to use this data pack for a custom map, you have to credit the creator, PmkExpert, in a visible way.
# Contact on Reddit for any question: https://www.reddit.com/user/Pmk23
# Visit Planet Minecraft for the latest updates: https://www.planetminecraft.com/member/pmk/
# Social Medias: https://twitter.com/PmkExpert https://www.youtube.com/user/Pmkexpert